@@ -1,7 +1,7 @@
 """
 *common tools used throughout package*
 """
-from .set_of_files import set_of_files
+# from .set_of_files import set_of_files
 from .keyword_lookup import keyword_lookup
 from .detector_lookup import detector_lookup
 from .create_dispersion_map import create_dispersion_map
@@ -14,3 +14,4 @@ from .detect_order_edges import detect_order_edges
 from .filenamer import filenamer
 from .dispersion_map_to_pixel_arrays import dispersion_map_to_pixel_arrays
 from .subtract_background import subtract_background
+from .subtract_sky import subtract_sky
