@@ -10,31 +10,28 @@
     September 10, 2020
 """
 ################# GLOBAL IMPORTS ####################
-from astropy.table import Table
-from astropy.io import fits
+
+
 from soxspipe.commonutils.toolkit import read_spectral_format
 from soxspipe.commonutils.toolkit import cut_image_slice
-import pandas as pd
 from soxspipe.commonutils.dispersion_map_to_pixel_arrays import dispersion_map_to_pixel_arrays
 from fundamentals.renderer import list_of_dictionaries
 import collections
-from astropy.visualization import hist
-from astropy.stats import sigma_clip, mad_std
-from scipy.optimize import curve_fit
+
+
 from random import random
-from scipy.signal import find_peaks
-from astropy.modeling import models, fitting
-from astropy.stats import mad_std
-import matplotlib.pyplot as plt
+
+
 from soxspipe.commonutils.filenamer import filenamer
 from soxspipe.commonutils.polynomials import chebyshev_xy_polynomial, chebyshev_order_xy_polynomials
-import numpy as np
+
 from os.path import expanduser
 from soxspipe.commonutils import detector_lookup
 from soxspipe.commonutils import keyword_lookup
 from fundamentals import tools
 from builtins import object
 import sys
+import math
 import os
 from io import StringIO
 import copy
@@ -49,19 +46,19 @@ class _base_detect(object):
             self,
             pixelList,
             order,
-            y_deg,
-            xCol,
-            yCol,
-            exponents_included=False):
+            axisBDeg,
+            axisACol,
+            axisBCol,
+            exponentsIncluded=False):
         """*iteratively fit the dispersion map polynomials to the data, clipping residuals with each iteration*
 
         **Key Arguments:**
             - ``pixelList`` -- data-frame group containing x,y pixel array
             - ``order`` -- the order to fit
-            - ``y_deg`` -- degree for polynomial to fit
-            - ``xCol`` -- name of x column
-            - ``yCol`` -- name of y column
-            - ``exponents_included`` -- the exponents have already been calculated in the dataframe so no need to regenerate. Default *False*
+            - ``axisBDeg`` -- degree for polynomial to fit
+            - ``axisACol`` -- name of columns containing axis to be fitted
+            - ``axisBCol`` -- name of columns containing free axis (values known)
+            - ``exponentsIncluded`` -- the exponents have already been calculated in the dataframe so no need to regenerate. Default *False*
 
         **Return:**
             - ``coeffs`` -- the coefficients of the polynomial fit
@@ -69,18 +66,22 @@ class _base_detect(object):
         """
         self.log.debug('starting the ``fit_order_polynomial`` method')
 
+        import numpy as np
+        from astropy.stats import sigma_clip
+        from scipy.optimize import curve_fit
+
         arm = self.arm
-        self.y_deg = y_deg
+        self.axisBDeg = axisBDeg
 
         clippedCount = 1
 
         poly = chebyshev_xy_polynomial(
-            log=self.log, yCol=yCol, y_deg=y_deg, exponents_included=exponents_included).poly
+            log=self.log, axisBCol=axisBCol, axisBDeg=self.axisBDeg, exponentsIncluded=exponentsIncluded).poly
 
         clippingSigma = self.recipeSettings[
             "poly-fitting-residual-clipping-sigma"]
         clippingIterationLimit = self.recipeSettings[
-            "clipping-iteration-limit"]
+            "poly-clipping-iteration-limit"]
 
         iteration = 0
         mask = (pixelList['order'] == order)
@@ -92,12 +93,12 @@ class _base_detect(object):
             startCount = len(pixelListFiltered.index)
             iteration += 1
             # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLY
-            coeff = np.ones((y_deg + 1))
+            coeff = np.ones((self.axisBDeg + 1))
             # NOTE X AND Y COLUMN ARE CORRECLY IN xdata AND ydata - WANT TO
             # FIND X (UNKNOWN) WRT Y (KNOWNN)
             try:
                 coeff, pcov_x = curve_fit(
-                    poly, xdata=pixelListFiltered, ydata=pixelListFiltered[xCol].values, p0=coeff)
+                    poly, xdata=pixelListFiltered, ydata=pixelListFiltered[axisACol].values, p0=coeff)
             except TypeError as e:
                 # REMOVE THIS ORDER FROM PIXEL LIST
                 pixelList.drop(index=pixelList[mask].index, inplace=True)
@@ -109,16 +110,15 @@ class _base_detect(object):
             res, res_mean, res_std, res_median, xfit = self.calculate_residuals(
                 orderPixelTable=pixelListFiltered,
                 coeff=coeff,
-                y_deg=y_deg,
-                xCol=xCol,
-                yCol=yCol)
+                axisACol=axisACol,
+                axisBCol=axisBCol)
 
             pixelList.loc[mask, "x_fit_res"] = res
             pixelList.loc[mask, "x_fit"] = xfit
 
             # SIGMA-CLIP THE DATA
             masked_residuals = sigma_clip(
-                res, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
+                res, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc='mad_std')
             pixelList.loc[mask, "mask"] = masked_residuals.mask
 
             # REMOVE FILTERED ROWS FROM DATA FRAME
@@ -136,20 +136,16 @@ class _base_detect(object):
     def fit_global_polynomial(
             self,
             pixelList,
-            y_deg,
-            order_deg,
-            xCol="cont_x",
-            yCol="cont_y",
+            axisACol="cont_x",
+            axisBCol="cont_y",
             orderCol="order",
-            exponents_included=False,
-            write_QCs=False):
-        """*iteratively fit the global polynomial to the data, clipping residuals with each iteration*
+            exponentsIncluded=False,
+            writeQCs=False):
+        """*iteratively fit the global polynomial to the data, fitting axisA as a function of axisB, clipping residuals with each iteration*
 
         **Key Arguments:**
             - ``pixelList`` -- data-frame group containing x,y pixel array
-            - ``y_deg`` -- degree for polynomial to fit y-values
-            - ``order_deg`` -- degree for polynomial to fit order-values
-            - ``exponents_included`` -- the exponents have already been calculated in the dataframe so no need to regenerate. Default *False*
+            - ``exponentsIncluded`` -- the exponents have already been calculated in the dataframe so no need to regenerate. Default *False*
 
         **Return:**
             - ``coeffs`` -- the coefficients of the polynomial fit
@@ -157,16 +153,20 @@ class _base_detect(object):
         """
         self.log.debug('starting the ``fit_global_polynomial`` method')
 
+        import numpy as np
+        from astropy.stats import sigma_clip
+        from scipy.optimize import curve_fit
+
         arm = self.arm
 
         clippedCount = 1
 
-        poly = chebyshev_order_xy_polynomials(log=self.log, yCol=yCol, orderCol=orderCol, order_deg=order_deg, y_deg=y_deg, exponents_included=exponents_included).poly
+        poly = chebyshev_order_xy_polynomials(log=self.log, axisBCol=axisBCol, orderCol=orderCol, orderDeg=self.orderDeg, axisBDeg=self.axisBDeg, axisB=self.axisB, exponentsIncluded=exponentsIncluded).poly
 
         clippingSigma = self.recipeSettings[
             "poly-fitting-residual-clipping-sigma"]
         clippingIterationLimit = self.recipeSettings[
-            "clipping-iteration-limit"]
+            "poly-clipping-iteration-limit"]
 
         iteration = 0
 
@@ -174,10 +174,10 @@ class _base_detect(object):
             startCount = len(pixelList.index)
             iteration += 1
             # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLY
-            coeff = np.ones((self.yDeg + 1) * (self.orderDeg + 1))
+            coeff = np.ones((self.axisBDeg + 1) * (self.orderDeg + 1))
             try:
                 coeff, pcov_x = curve_fit(
-                    poly, xdata=pixelList, ydata=pixelList[xCol].values, p0=coeff)
+                    poly, xdata=pixelList, ydata=pixelList[axisACol].values, p0=coeff)
             except TypeError as e:
                 # REMOVE THIS ORDER FROM PIXEL LIST
                 coeff = None
@@ -188,18 +188,16 @@ class _base_detect(object):
             res, res_mean, res_std, res_median, xfit = self.calculate_residuals(
                 orderPixelTable=pixelList,
                 coeff=coeff,
-                y_deg=y_deg,
-                order_deg=order_deg,
                 orderCol=orderCol,
-                xCol=xCol,
-                yCol=yCol)
+                axisACol=axisACol,
+                axisBCol=axisBCol)
 
-            pixelList["x_fit_res"] = res
-            pixelList["x_fit"] = xfit
+            pixelList[f"{axisACol}_fit_res"] = res
+            pixelList[f"{axisACol}_fit"] = xfit
 
             # SIGMA-CLIP THE DATA
             masked_residuals = sigma_clip(
-                res, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
+                res, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc='mad_std')
             pixelList["mask"] = masked_residuals.mask
 
             # REMOVE FILTERED ROWS FROM DATA FRAME
@@ -214,12 +212,10 @@ class _base_detect(object):
         res, res_mean, res_std, res_median, xfit = self.calculate_residuals(
             orderPixelTable=pixelList,
             coeff=coeff,
-            y_deg=y_deg,
-            order_deg=order_deg,
             orderCol=orderCol,
-            xCol=xCol,
-            yCol=yCol,
-            write_QCs=write_QCs)
+            axisACol=axisACol,
+            axisBCol=axisBCol,
+            writeQCs=writeQCs)
 
         self.log.debug('completed the ``fit_global_polynomial`` method')
         return coeff, pixelList
@@ -228,23 +224,19 @@ class _base_detect(object):
             self,
             orderPixelTable,
             coeff,
-            y_deg,
-            xCol,
-            yCol,
+            axisACol,
+            axisBCol,
             orderCol=False,
-            order_deg=False,
-            write_QCs=False):
+            writeQCs=False):
         """*calculate residuals of the polynomial fits against the observed line postions*
 
         **Key Arguments:**
             - ``orderPixelTable`` -- data-frame containing pixel list for given order
             - ``coeff`` -- the coefficients of the fitted polynomial
-            - ``y_deg`` -- degree for polynomial to fit y-values
-            - ``xCol`` -- name of x-pixel column
-            - ``yCol`` -- name of y-pixel column
+            - ``axisACol`` -- name of x-pixel column
+            - ``axisBCol`` -- name of y-pixel column
             - ``orderCol`` -- name of the order column (global fits only)
-            - ``order_deg`` -- degree for polynomial to fit order-values (global fits only)
-            - ``write_QCs`` -- write the QCs to dataframe? Default *False*
+            - ``writeQCs`` -- write the QCs to dataframe? Default *False*
 
         **Return:**
             - ``res`` -- x residuals
@@ -255,26 +247,28 @@ class _base_detect(object):
         """
         self.log.debug('starting the ``calculate_residuals`` method')
 
+        import numpy as np
+
         arm = self.arm
 
         poly = chebyshev_order_xy_polynomials(
-            log=self.log, yCol=yCol, orderCol=orderCol, order_deg=self.orderDeg, y_deg=self.yDeg).poly
+            log=self.log, axisBCol=axisBCol, orderCol=orderCol, orderDeg=self.orderDeg, axisBDeg=self.axisBDeg).poly
 
         # CALCULATE RESIDUALS BETWEEN GAUSSIAN PEAK LINE POSITIONS AND POLY
         # FITTED POSITIONS
         xfit = poly(
             orderPixelTable, *coeff)
-        res = abs(xfit - orderPixelTable[xCol].values)
+        res = xfit - orderPixelTable[axisACol].values
 
         # GET UNIQUE VALUES IN COLUMN
         uniqueorders = len(orderPixelTable['order'].unique())
 
         # CALCULATE COMBINED RESIDUALS AND STATS
-        res_mean = np.mean(res)
-        res_std = np.std(res)
-        res_median = np.median(res)
+        res_mean = np.ma.mean(res)
+        res_std = np.ma.std(res)
+        res_median = np.ma.median(res)
 
-        if write_QCs:
+        if writeQCs:
             utcnow = datetime.utcnow()
             utcnow = utcnow.strftime("%Y-%m-%dT%H:%M:%S")
 
@@ -342,6 +336,7 @@ class _base_detect(object):
             - ``order_table_path`` -- path to the order table file
         """
         from astropy.table import Table
+        from astropy.io import fits
         self.log.debug('starting the ``write_order_table_to_file`` method')
 
         arm = self.arm
@@ -349,15 +344,28 @@ class _base_detect(object):
 
         # DETERMINE WHERE TO WRITE THE FILE
         home = expanduser("~")
-        outDir = self.settings["intermediate-data-root"].replace("~", home)
+        if self.binx > 1 or self.biny > 1:
+            outDir = self.settings["intermediate-data-root"] + "/tmp"
+        else:
+            outDir = self.settings["intermediate-data-root"].replace("~", home) + f"/product/{self.recipeName}"
+            outDir = outDir.replace("//", "/")
+        # Recursively create missing directories
+        if not os.path.exists(outDir):
+            os.makedirs(outDir)
 
-        filename = filenamer(
-            log=self.log,
-            frame=frame,
-            settings=self.settings
-        )
+        if not self.sofName:
+            filename = filenamer(
+                log=self.log,
+                frame=frame,
+                settings=self.settings
+            )
+
+        else:
+            filename = self.sofName + ".fits"
         filename = filename.replace("MFLAT", "FLAT")
-        filename = filename.split("FLAT")[0] + "ORDER_LOCATIONS.fits"
+
+        if "FLAT" in filename.upper():
+            filename = filename.upper().split("FLAT")[0] + "ORDER_LOCATIONS.fits"
 
         order_table_path = f"{outDir}/{filename}"
 
@@ -406,6 +414,9 @@ class detect_continuum(_base_detect):
         - ``recipeName`` -- the recipe name as given in the settings dictionary
         - ``qcTable`` -- the data frame to collect measured QC metrics 
         - ``productsTable`` -- the data frame to collect output products
+        - ``sofName`` ---- name of the originating SOF file
+        - ``binx`` -- binning in x-axis
+        - ``biny`` -- binning in y-axis
 
     **Usage:**
 
@@ -432,7 +443,10 @@ class detect_continuum(_base_detect):
             settings=False,
             recipeName=False,
             qcTable=False,
-            productsTable=False
+            productsTable=False,
+            sofName=False,
+            binx=1,
+            biny=1
     ):
         self.log = log
         log.debug("instansiating a new 'detect_continuum' object")
@@ -446,6 +460,9 @@ class detect_continuum(_base_detect):
         self.dispersion_map = dispersion_map
         self.qc = qcTable
         self.products = productsTable
+        self.sofName = sofName
+        self.binx = binx
+        self.biny = biny
 
         # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
         # FOLDER
@@ -455,6 +472,7 @@ class detect_continuum(_base_detect):
         ).get
         self.arm = pinholeFlat.header[self.kw("SEQ_ARM")]
         self.dateObs = pinholeFlat.header[self.kw("DATE_OBS")]
+        self.inst = pinholeFlat.header[self.kw("INSTRUME")]
 
         # DETECTOR PARAMETERS LOOKUP OBJECT
         self.detectorParams = detector_lookup(
@@ -463,7 +481,7 @@ class detect_continuum(_base_detect):
         ).get(self.arm)
 
         # DEG OF THE POLYNOMIALS TO FIT THE ORDER CENTRE LOCATIONS
-        self.yDeg = self.recipeSettings["y-deg"]
+        self.axisBDeg = self.recipeSettings["disp-axis-deg"]
         self.orderDeg = self.recipeSettings["order-deg"]
 
         return None
@@ -477,6 +495,9 @@ class detect_continuum(_base_detect):
         """
         self.log.debug('starting the ``get`` method')
 
+        import numpy as np
+        import pandas as pd
+
         arm = self.arm
 
         # READ THE SPECTRAL FORMAT TABLE TO DETERMINE THE LIMITS OF THE TRACES
@@ -493,18 +514,34 @@ class detect_continuum(_base_detect):
         self.sliceLength = self.recipeSettings["slice-length"]
         self.peakSigmaLimit = self.recipeSettings["peak-sigma-limit"]
 
+        if self.inst == "SOXS":
+            self.axisA = "y"
+            self.axisB = "x"
+            coeff_dict = {"degorder_cent": self.orderDeg,
+                          "degx_cent": self.axisBDeg}
+
+        elif self.inst == "XSHOOTER":
+            self.axisA = "x"
+            self.axisB = "y"
+            coeff_dict = {"degorder_cent": self.orderDeg,
+                          "degy_cent": self.axisBDeg}
+
         # PREP LISTS WITH NAN VALUE IN CONT_X AND CONT_Y BEFORE FITTING
-        orderPixelTable['cont_x'] = np.nan
-        orderPixelTable['cont_y'] = np.nan
+        orderPixelTable[f'cont_{self.axisA}'] = np.nan
+        orderPixelTable[f'cont_{self.axisB}'] = np.nan
 
         # FOR EACH ORDER, FOR EACH PIXEL POSITION SAMPLE, FIT A 1D GAUSSIAN IN
         # CROSS-DISPERSION DIRECTTION. RETURN PEAK POSTIONS
+        from soxspipe.commonutils.toolkit import quicklook_image
+        quicklook_image(
+            log=self.log, CCDObject=self.pinholeFlat, show=False, ext='data', stdWindow=3, title=False, surfacePlot=True)
         orderPixelTable = orderPixelTable.apply(
             self.fit_1d_gaussian_to_slice, axis=1)
         allLines = len(orderPixelTable.index)
         # DROP ROWS WITH NAN VALUES
         orderPixelTable.dropna(axis='index', how='any',
                                subset=['cont_x'], inplace=True)
+
         foundLines = len(orderPixelTable.index)
         percent = 100 * foundLines / allLines
         print(f"\tContinuum found in {foundLines} out of {allLines} order slices ({percent:2.0f}%)")
@@ -513,12 +550,12 @@ class detect_continuum(_base_detect):
         uniqueOrders = orderPixelTable['order'].unique()
 
         orderLocations = {}
-        orderPixelTable['x_fit'] = np.nan
-        orderPixelTable['x_fit_res'] = np.nan
+        orderPixelTable[f'cont_{self.axisA}_fit'] = np.nan
+        orderPixelTable[f'cont_{self.axisA}_fit_res'] = np.nan
 
         # SETUP EXPONENTS AHEAD OF TIME - SAVES TIME ON POLY FITTING
-        for i in range(0, self.yDeg + 1):
-            orderPixelTable[f"y_pow_{i}"] = orderPixelTable["cont_y"].pow(i)
+        for i in range(0, self.axisBDeg + 1):
+            orderPixelTable[f"{self.axisB}_pow_{i}"] = orderPixelTable[f"cont_{self.axisB}"].pow(i)
         for i in range(0, self.orderDeg + 1):
             orderPixelTable[f"order_pow_{i}"] = orderPixelTable["order"].pow(i)
 
@@ -527,20 +564,33 @@ class detect_continuum(_base_detect):
         # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
         coeff, orderPixelTable = self.fit_global_polynomial(
             pixelList=orderPixelTable,
-            y_deg=self.yDeg,
-            order_deg=self.orderDeg,
-            exponents_included=True,
-            write_QCs=True
+            axisACol=f"cont_{self.axisA}",
+            axisBCol=f"cont_{self.axisB}",
+            exponentsIncluded=True,
+            writeQCs=True
         )
 
-        # orderLocations[o] = coeff
-        coeff_dict = {"degorder_cent": self.orderDeg,
-                      "degy_cent": self.yDeg}
         n_coeff = 0
         for i in range(0, self.orderDeg + 1):
-            for j in range(0, self.yDeg + 1):
+            for j in range(0, self.axisBDeg + 1):
                 coeff_dict[f'cent_{i}{j}'] = coeff[n_coeff]
                 n_coeff += 1
+
+        # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
+        coeff, orderPixelTable = self.fit_global_polynomial(
+            pixelList=orderPixelTable,
+            axisACol="stddev",
+            axisBCol=f"cont_{self.axisB}",
+            exponentsIncluded=True
+        )
+        # orderLocations[o] = coeff
+        coeff_dict["degorder_std"] = self.orderDeg,
+        coeff_dict[f"deg{self.axisB}_std"] = self.axisBDeg
+        n_coeff = 0
+        for i in range(0, self.orderDeg + 1):
+            for j in range(0, self.axisBDeg + 1):
+                coeff_dict[f'std_{i}{j}'] = coeff[n_coeff]
+                n_coeff += 1
         coeffColumns = coeff_dict.keys()
         dataSet = list_of_dictionaries(
             log=self.log,
@@ -577,8 +627,8 @@ class detect_continuum(_base_detect):
             "file_path": plotPath
         }, ignore_index=True)
 
-        mean_res = np.mean(np.abs(orderPixelTable['x_fit_res'].values))
-        std_res = np.std(np.abs(orderPixelTable['x_fit_res'].values))
+        mean_res = np.mean(np.abs(orderPixelTable[f'cont_{self.axisA}_fit_res'].values))
+        std_res = np.std(np.abs(orderPixelTable[f'cont_{self.axisA}_fit_res'].values))
 
         # WRITE OUT THE FITS TO THE ORDER CENTRE TABLE
         order_table_path = self.write_order_table_to_file(
@@ -604,20 +654,65 @@ class detect_continuum(_base_detect):
         """
         self.log.debug('starting the ``create_pixel_arrays`` method')
 
+        import numpy as np
+        import pandas as pd
+
         # READ ORDER SAMPLING RESOLUTION FROM SETTINGS
         sampleCount = self.settings[
             "soxs-order-centre"]["order-sample-count"]
 
+        # FIND THE PIXEL RANGES FOR ALL ORDERS
+        myDict = {
+            "order": np.asarray([]),
+            "wavelength": np.asarray([]),
+            "slit_position": np.asarray([])
+        }
+        for o, wmin, wmax in zip(orderNums, waveLengthMin, waveLengthMax):
+            wlArray = np.array([wmin, wmax])
+            myDict["wavelength"] = np.append(myDict["wavelength"], wlArray)
+            myDict["order"] = np.append(
+                myDict["order"], np.ones(len(wlArray)) * o)
+            myDict["slit_position"] = np.append(
+                myDict["slit_position"], np.zeros(len(wlArray)))
+        orderPixelTable = pd.DataFrame(myDict)
+        orderPixelTable = dispersion_map_to_pixel_arrays(
+            log=self.log,
+            dispersionMapPath=self.dispersion_map,
+            orderPixelTable=orderPixelTable,
+            removeOffDetectorLocation=False
+        )
+        orderPixelRanges = []
+        if self.inst == "SOXS":
+            axis = "x"
+            rowCol = "columns"
+        else:
+            axis = "y"
+            rowCol = "rows"
+
+        for o in orderNums:
+            amin = orderPixelTable.loc[orderPixelTable["order"] == o, f"fit_{axis}"].min()
+            amax = orderPixelTable.loc[orderPixelTable["order"] == o, f"fit_{axis}"].max()
+            if amin < 0:
+                amin = 0
+            if amax > self.detectorParams["science-pixels"][rowCol]["end"]:
+                amax = self.detectorParams["science-pixels"][rowCol]["end"]
+
+            arange = amax - amin
+            orderPixelRanges.append(arange)
+
+        smallestRange = min(orderPixelRanges)
+        samplePixelSep = int(smallestRange / sampleCount)
+
         # CREATE THE WAVELENGTH/ORDER ARRAYS TO BE CONVERTED TO PIXELS
         myDict = {
             "order": np.asarray([]),
             "wavelength": np.asarray([]),
             "slit_position": np.asarray([])
         }
-        for o, wmin, wmax in zip(orderNums, waveLengthMin, waveLengthMax):
-
+        for o, wmin, wmax, pixelRange in zip(orderNums, waveLengthMin, waveLengthMax, orderPixelRanges):
+            orderSampleCount = int(pixelRange / samplePixelSep)
             wlArray = np.arange(
-                wmin, wmax, (wmax - wmin) / sampleCount)
+                wmin, wmax, (wmax - wmin) / orderSampleCount)
             myDict["wavelength"] = np.append(myDict["wavelength"], wlArray)
             myDict["order"] = np.append(
                 myDict["order"], np.ones(len(wlArray)) * o)
@@ -647,19 +742,32 @@ class detect_continuum(_base_detect):
         """
         self.log.debug('starting the ``fit_1d_gaussian_to_slice`` method')
 
+        import numpy as np
+        from astropy.stats import mad_std
+        from astropy.modeling import models, fitting
+        from scipy.signal import find_peaks
+
         # CLIP OUT A SLICE TO INSPECT CENTRED AT POSITION
         halfSlice = self.sliceLength / 2
 
-        slice = cut_image_slice(log=self.log, frame=self.pinholeFlat,
-                                width=1, length=self.sliceLength, x=pixelPostion["fit_x"], y=pixelPostion["fit_y"], median=True, plot=False)
+        if self.inst == "SOXS":
+            sliceAxis = "y"
+            sliceAntiAxis = "x"
+        else:
+            sliceAxis = "x"
+            sliceAntiAxis = "y"
+
+        slice, slice_length_offset, slice_width_centre = cut_image_slice(log=self.log, frame=self.pinholeFlat,
+                                                                         width=1, length=self.sliceLength, x=pixelPostion["fit_x"], y=pixelPostion["fit_y"], sliceAxis=sliceAxis, median=True, plot=False)
 
         if slice is None:
-            pixelPostion["cont_x"] = np.nan
-            pixelPostion["cont_y"] = np.nan
+            pixelPostion[f"cont_{self.axisA}"] = np.nan
+            pixelPostion[f"cont_{self.axisB}"] = np.nan
             return pixelPostion
 
         # CHECK THE SLICE POINTS IF NEEDED
         if 1 == 0:
+            import matplotlib.pyplot as plt
             x = np.arange(0, len(slice))
             plt.figure(figsize=(8, 5))
             plt.plot(x, slice, 'ko')
@@ -669,12 +777,12 @@ class detect_continuum(_base_detect):
 
         # EVALUATING THE MEAN AND STD-DEV FOR PEAK FINDING - REMOVES SLICE
         # CONTAINING JUST NOISE
-        median_r = np.median(slice)
+        median_r = np.ma.median(slice)
         std_r = mad_std(slice)
 
         if not median_r:
-            pixelPostion["cont_x"] = np.nan
-            pixelPostion["cont_y"] = np.nan
+            pixelPostion[f"cont_{self.axisA}"] = np.nan
+            pixelPostion[f"cont_{self.axisB}"] = np.nan
             return pixelPostion
 
         peaks, _ = find_peaks(slice, height=median_r +
@@ -690,8 +798,8 @@ class detect_continuum(_base_detect):
                 plt.xlabel('Position')
                 plt.ylabel('Flux')
                 plt.show()
-            pixelPostion["cont_x"] = np.nan
-            pixelPostion["cont_y"] = np.nan
+            pixelPostion[f"cont_{self.axisA}"] = np.nan
+            pixelPostion[f"cont_{self.axisB}"] = np.nan
             return pixelPostion
 
         # FIT THE DATA USING A 1D GAUSSIAN - USING astropy.modeling
@@ -703,19 +811,22 @@ class detect_continuum(_base_detect):
 
         # NOW FIT
         g = fit_g(g_init, np.arange(0, len(slice)), slice)
-        pixelPostion["cont_x"] = g.mean + \
-            max(0, int(pixelPostion["fit_x"] - halfSlice))
-        pixelPostion["cont_y"] = pixelPostion["fit_y"]
+        pixelPostion[f"cont_{sliceAxis}"] = g.mean.value + \
+            max(0, slice_length_offset)
+        pixelPostion[f"cont_{sliceAntiAxis}"] = slice_width_centre
+        pixelPostion["amplitude"] = g.amplitude.value
+        pixelPostion["stddev"] = g.stddev.value
 
-        # PRINT A FEW PLOTS IF NEEDED - GUASSIAN FIT OVERLAYED
-        if 1 == 0 and random() < 0.02:
+        # PRINT A FEW PLOTS IF NEEDED - GAUSSIAN FIT OVERLAYED
+        if 1 == 0 and random() < 0.02 and pixelPostion["order"] == 11:
             x = np.arange(0, len(slice))
             plt.figure(figsize=(8, 5))
             plt.plot(x, slice, 'ko')
             plt.xlabel('Position')
             plt.ylabel('Flux')
-            guassx = np.arange(0, max(x), 0.05)
-            plt.plot(guassx, g(guassx), label='Gaussian')
+            gaussx = np.arange(0, max(x), 0.05)
+            plt.plot(gaussx, g(gaussx), label=f'Mean = {g.mean.value:0.2f}, std = {g.stddev.value:0.2f}, cont_{self.axisA} = {pixelPostion[f"cont_{self.axisA}"]:0.2f}, fit_{self.axisA} = {pixelPostion[f"fit_{self.axisA}"]:0.2f},cont_{self.axisB} = {pixelPostion[f"cont_{self.axisB}"]:0.2f},fit_y = {pixelPostion[f"fit_{self.axisB}"]:0.2f}')
+            plt.legend()
             plt.show()
 
         self.log.debug('completed the ``fit_1d_gaussian_to_slice`` method')
@@ -737,48 +848,71 @@ class detect_continuum(_base_detect):
         """
         self.log.debug('starting the ``plot_results`` method')
 
+        import numpy as np
+        import pandas as pd
+        import matplotlib.pyplot as plt
+
         arm = self.arm
 
         # a = plt.figure(figsize=(40, 15))
-        if arm == "UVB":
+        if arm == "UVB" or self.inst == "SOXS":
             fig = plt.figure(figsize=(6, 13.5), constrained_layout=True)
+            # CREATE THE GID OF AXES
+            gs = fig.add_gridspec(6, 4)
+            toprow = fig.add_subplot(gs[0:2, :])
+            midrow = fig.add_subplot(gs[2:4, :])
+            bottomleft = fig.add_subplot(gs[4:5, 0:2])
+            bottomright = fig.add_subplot(gs[4:5, 2:])
+            fwhmaxis = fig.add_subplot(gs[5:6, :])
         else:
-            fig = plt.figure(figsize=(6, 11), constrained_layout=True)
-        gs = fig.add_gridspec(6, 4)
-
-        # CREATE THE GID OF AXES
-        toprow = fig.add_subplot(gs[0:2, :])
-        midrow = fig.add_subplot(gs[2:4, :])
-        bottomleft = fig.add_subplot(gs[4:, 0:2])
-        bottomright = fig.add_subplot(gs[4:, 2:])
+            fig = plt.figure(figsize=(6, 12), constrained_layout=True)
+            # CREATE THE GID OF AXES
+            gs = fig.add_gridspec(7, 4)
+            toprow = fig.add_subplot(gs[0:2, :])
+            midrow = fig.add_subplot(gs[2:4, :])
+            bottomleft = fig.add_subplot(gs[4:6, 0:2])
+            bottomright = fig.add_subplot(gs[4:6, 2:])
+            fwhmaxis = fig.add_subplot(gs[6:7, :])
 
         # ROTATE THE IMAGE FOR BETTER LAYOUT
-        rotatedImg = np.rot90(self.pinholeFlat.data, 1)
+        rotatedImg = self.pinholeFlat.data
+        if self.axisA == "x":
+            rotatedImg = np.rot90(rotatedImg, 1)
+            rotatedImg = np.flipud(rotatedImg)
         toprow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
         toprow.set_title(
             "1D guassian peak positions (post-clipping)", fontsize=10)
-        x = np.ones(len(orderPixelTable.index)) * \
-            self.pinholeFlat.data.shape[1] - orderPixelTable['cont_x'].values
         toprow.scatter(orderPixelTable[
-                       'cont_y'].values, x, marker='x', c='red', s=4)
+                       f'cont_{self.axisB}'].values, orderPixelTable[f'cont_{self.axisA}'].values, marker='o', c='red', s=1, alpha=0.6)
+
         # toprow.set_yticklabels([])
         # toprow.set_xticklabels([])
-        toprow.set_ylabel("x-axis", fontsize=8)
-        toprow.set_xlabel("y-axis", fontsize=8)
+
+        toprow.set_ylabel(f"{self.axisA}-axis", fontsize=12)
+        toprow.set_xlabel(f"{self.axisB}-axis", fontsize=12)
         toprow.tick_params(axis='both', which='major', labelsize=9)
+        if self.axisA == "x":
+            toprow.invert_yaxis()
 
         midrow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
         midrow.set_title(
             "order-location fit solutions", fontsize=10)
-        ylinelist = np.arange(0, self.pinholeFlat.data.shape[0], 3)
+        if self.axisB == "y":
+            axisALength = self.pinholeFlat.data.shape[1]
+            axisBLength = self.pinholeFlat.data.shape[0]
+        elif self.axisB == "x":
+            axisALength = self.pinholeFlat.data.shape[0]
+            axisBLength = self.pinholeFlat.data.shape[1]
+        axisBlinelist = np.arange(0, axisBLength, 3)
 
         poly = chebyshev_order_xy_polynomials(
-            log=self.log, yCol="y", orderCol="order", order_deg=self.orderDeg, y_deg=self.yDeg).poly
+            log=self.log, axisBCol=self.axisB, orderCol="order", orderDeg=self.orderDeg, axisBDeg=self.axisBDeg).poly
         for index, row in orderPolyTable.iterrows():
-            coeff = [float(v) for k, v in row.items() if "cent_" in k]
+            cent_coeff = [float(v) for k, v in row.items() if "cent_" in k]
+            std_coeff = [float(v) for k, v in row.items() if "std_" in k]
         uniqueOrders = orderPixelTable['order'].unique()
         # CREATE DATA FRAME FROM A DICTIONARY OF LISTS
-        myDict = {"y": ylinelist}
+        myDict = {f"{self.axisB}": axisBlinelist}
         df = pd.DataFrame(myDict)
         ymin = []
         ymax = []
@@ -786,25 +920,28 @@ class detect_continuum(_base_detect):
         xmax = []
         for o in uniqueOrders:
             df["order"] = o
-            xfit = poly(df, *coeff)
-            xfit = np.ones(len(xfit)) * \
-                self.pinholeFlat.data.shape[1] - xfit
-            xfit, yfit = zip(
-                *[(x, y) for x, y in zip(xfit, ylinelist) if x > 0 and x < (self.pinholeFlat.data.shape[1]) - 10])
-            l = midrow.plot(yfit, xfit)
+            xfit = poly(df, *cent_coeff)
+            stdfit = poly(df, *std_coeff)
+            xfit, yfit, stdfit, lower, upper = zip(
+                *[(x, y, std, x - 3 * std, x + 3 * std) for x, y, std in zip(xfit, axisBlinelist, stdfit) if x > 0 and x < (axisALength) - 10])
+            # lower = xfit - 3 * stdfit
+            # upper = xfit + 3 * stdfit
+            c = midrow.plot(yfit, xfit, linewidth=0.7)
+            midrow.fill_between(yfit, lower, upper, color=c[0].get_color(), alpha=0.3)
+
             ymin.append(min(yfit))
             ymax.append(max(yfit))
-            xmin.append(self.pinholeFlat.data.shape[1] - max(xfit))
-            xmax.append(self.pinholeFlat.data.shape[1] - min(xfit))
-            midrow.text(yfit[10], xfit[10] - 20, int(o), fontsize=6, c="white", verticalalignment='bottom')
+            xmin.append(axisALength - max(xfit))
+            xmax.append(axisALength - min(xfit))
+            midrow.text(yfit[10], xfit[10] + 10, int(o), fontsize=6, c="white", verticalalignment='bottom')
 
         # CREATE DATA FRAME FROM A DICTIONARY OF LISTS
         orderMetaTable = {
             "order": uniqueOrders,
-            "ymin": ymin,
-            "ymax": ymax,
-            "xmin": xmin,
-            "xmax": xmax,
+            f"{self.axisB}min": ymin,
+            f"{self.axisB}max": ymax,
+            f"{self.axisA}min": xmin,
+            f"{self.axisA}max": xmax,
         }
         orderMetaTable = pd.DataFrame(orderMetaTable)
 
@@ -813,45 +950,64 @@ class detect_continuum(_base_detect):
         # midrow.scatter(yfit, xfit, marker='x', c='blue', s=4)
         # midrow.set_yticklabels([])
         # midrow.set_xticklabels([])
-        midrow.set_ylabel("x-axis", fontsize=8)
-        midrow.set_xlabel("y-axis", fontsize=8)
+        midrow.set_ylabel(f"{self.axisA}-axis", fontsize=12)
+        midrow.set_xlabel(f"{self.axisB}-axis", fontsize=12)
         midrow.tick_params(axis='both', which='major', labelsize=9)
 
+        if self.axisA == "x":
+            midrow.invert_yaxis()
+            midrow.set_ylim(0, axisALength)
+
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomleft.scatter(orderPixelTable['cont_x'].values, orderPixelTable[
-                           'x_fit_res'].values, alpha=0.2, s=1)
-        bottomleft.set_xlabel('x pixel position')
-        bottomleft.set_ylabel('x residual')
+        bottomleft.scatter(orderPixelTable[f'cont_{self.axisA}'].values, orderPixelTable[
+                           f'cont_{self.axisA}_fit_res'].values, alpha=0.2, s=1)
+        bottomleft.set_xlabel(f'{self.axisA} pixel position', fontsize=10)
+        bottomleft.set_ylabel(f'{self.axisA} residual', fontsize=10)
         bottomleft.tick_params(axis='both', which='major', labelsize=9)
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomright.scatter(orderPixelTable['cont_y'].values, orderPixelTable[
-                            'x_fit_res'].values, alpha=0.2, s=1)
-        bottomright.set_xlabel('y pixel position')
+        bottomright.scatter(orderPixelTable[f'cont_{self.axisB}'].values, orderPixelTable[
+                            f'cont_{self.axisA}_fit_res'].values, alpha=0.2, s=1)
+        bottomright.set_xlabel(f'{self.axisB} pixel position', fontsize=10)
         bottomright.tick_params(axis='both', which='major', labelsize=9)
         # bottomright.set_ylabel('x residual')
         bottomright.set_yticklabels([])
 
-        mean_res = np.mean(np.abs(orderPixelTable['x_fit_res'].values))
-        std_res = np.std(np.abs(orderPixelTable['x_fit_res'].values))
+        stdToFwhm = 2 * (2 * math.log(2))**0.5
+        fwhmaxis.scatter(orderPixelTable['wavelength'].values, orderPixelTable['stddev_fit'].values * stdToFwhm, alpha=0.2, s=1)
+        fwhmaxis.set_xlabel('wavelength (nm)', fontsize=10)
+        fwhmaxis.set_ylabel('FWHM (pixels)', fontsize=10)
+        fwhmaxis.set_ylim(0, orderPixelTable['stddev_fit'].max() * stdToFwhm)
+
+        mean_res = np.mean(np.abs(orderPixelTable[f'cont_{self.axisA}_fit_res'].values))
+        std_res = np.std(np.abs(orderPixelTable[f'cont_{self.axisA}_fit_res'].values))
 
         subtitle = f"mean res: {mean_res:2.2f} pix, res stdev: {std_res:2.2f}"
         fig.suptitle(f"traces of order-centre locations - pinhole flat-frame\n{subtitle}", fontsize=12)
 
         # plt.show()
-        filename = filenamer(
-            log=self.log,
-            frame=self.pinholeFlat,
-            settings=self.settings
-        )
-        filename = filename.split("FLAT")[0] + "ORDER_CENTRES_residuals.pdf"
+        if not self.sofName:
+            filename = filenamer(
+                log=self.log,
+                frame=self.pinholeFlat,
+                settings=self.settings
+            )
+            filename = filename.split("FLAT")[0] + "ORDER_CENTRES_residuals.pdf"
+        else:
+            filename = self.sofName + "_residuals.pdf"
 
         home = expanduser("~")
-        outDir = self.settings["intermediate-data-root"].replace("~", home)
+        outDir = self.settings["intermediate-data-root"].replace("~", home) + "/qc/pdf"
+        outDir = outDir.replace("//", "/")
+        # Recursively create missing directories
+        if not os.path.exists(outDir):
+            os.makedirs(outDir)
+
         filePath = f"{outDir}/{filename}"
-        plt.savefig(filePath)
+        plt.tight_layout()
+        plt.savefig(filePath, dpi=720)
 
         self.log.debug('completed the ``plot_results`` method')
         return filePath, orderMetaTable
