@@ -12,21 +12,14 @@
 from soxspipe.commonutils import keyword_lookup
 from os.path import expanduser
 import random
-from scipy.ndimage.filters import median_filter
-from scipy.signal import medfilt2d
 from soxspipe.commonutils.toolkit import quicklook_image
-from scipy.interpolate import BSpline, splrep
-from astropy import units as u
-from astropy.nddata import CCDData
-import matplotlib.pyplot as plt
-import numpy as np
-import numpy.ma as ma
+
 from soxspipe.commonutils.toolkit import unpack_order_table
 from fundamentals import tools
 from builtins import object
 import sys
 import math
-import pandas as pd
+
 import os
 os.environ['TERM'] = 'vt100'
 
@@ -94,6 +87,15 @@ class subtract_background(object):
         self.arm = frame.header[kw("SEQ_ARM")]
         self.dateObs = frame.header[kw("DATE_OBS")]
 
+        self.inst = frame.header[kw("INSTRUME")]
+
+        if self.inst == "SOXS":
+            self.axisA = "y"
+            self.axisB = "x"
+        elif self.inst == "XSHOOTER":
+            self.axisA = "x"
+            self.axisB = "y"
+
         quicklook_image(
             log=self.log, CCDObject=self.frame, show=False, ext='data', stdWindow=3, surfacePlot=True, title="Initial input frame needing scattered light subtraction")
 
@@ -108,16 +110,18 @@ class subtract_background(object):
         """
         self.log.debug('starting the ``subtract`` method')
 
+        import numpy as np
+
         kw = self.kw
-        imageType = self.frame.header[kw("DPR_TYPE").lower()].replace(",", "-")
-        imageTech = self.frame.header[kw("DPR_TECH").lower()].replace(",", "-")
-        imageCat = self.frame.header[kw("DPR_CATG").lower()].replace(",", "-")
+        imageType = self.frame.header[kw("DPR_TYPE")].replace(",", "-")
+        imageTech = self.frame.header[kw("DPR_TECH")].replace(",", "-")
+        imageCat = self.frame.header[kw("DPR_CATG")].replace(",", "-")
 
         print(f"\n# FITTING AND SUBTRACTING SCATTERED LIGHT BACKGROUND FROM {self.arm} {imageCat} {imageTech} {imageType} FRAME")
 
         # UNPACK THE ORDER TABLE
         orderPolyTable, orderPixelTable, orderMetaTable = unpack_order_table(
-            log=self.log, orderTablePath=self.orderTable, extend=1.5)
+            log=self.log, orderTablePath=self.orderTable, extend=4)
 
         originalMask = np.copy(self.frame.mask)
 
@@ -128,7 +132,7 @@ class subtract_background(object):
             log=self.log, CCDObject=self.frame, show=False, ext=None, surfacePlot=True, title="Initial input frame with order pixels masked")
 
         backgroundFrame = self.create_background_image(
-            rowFitOrder=self.settings['background-subtraction']['bsline-deg'], medianFilterSize=self.settings['background-subtraction']['median-filter-pixels'])
+            rowFitOrder=self.settings['background-subtraction']['bspline-deg'], medianFilterSize=self.settings['background-subtraction']['median-filter-pixels'])
 
         # REPLACE MASK
         self.frame.mask = originalMask
@@ -137,7 +141,7 @@ class subtract_background(object):
         backgroundSubtractedFrame.header = self.frame.header
 
         quicklook_image(
-            log=self.log, CCDObject=backgroundSubtractedFrame, show=False, ext='data')
+            log=self.log, CCDObject=backgroundFrame, show=False, ext='data', stdWindow=3, surfacePlot=True, title="Scattered Light Image")
 
         self.log.debug('completed the ``subtract`` method')
         return backgroundFrame, backgroundSubtractedFrame
@@ -152,20 +156,30 @@ class subtract_background(object):
         """
         self.log.debug('starting the ``mask_order_locations`` method')
 
+        import numpy as np
+
         # MASK DATA INSIDE OF ORDERS (EXPAND THE INNER-ORDER AREA IF NEEDED)
         uniqueOrders = orderPixelTable['order'].unique()
+        if self.axisA == "x":
+            axisALen = self.frame.data.shape[1]
+            axisBLen = self.frame.data.shape[0]
+        else:
+            axisALen = self.frame.data.shape[0]
+            axisBLen = self.frame.data.shape[1]
         expandEdges = 2
         for o in uniqueOrders:
-            ycoord = orderPixelTable.loc[
-                (orderPixelTable["order"] == o)]["ycoord"]
-            xcoord_edgeup = orderPixelTable.loc[(orderPixelTable["order"] == o)][
-                "xcoord_edgeup"] + expandEdges
-            xcoord_edgelow = orderPixelTable.loc[(orderPixelTable["order"] == o)][
-                "xcoord_edgelow"] - expandEdges
-            xcoord_edgelow, xcoord_edgeup, ycoord = zip(*[(x1, x2, y) for x1, x2, y in zip(xcoord_edgelow, xcoord_edgeup, ycoord) if x1 > 0 and x1 < self.frame.data.shape[
-                                                        1] and x2 > 0 and x2 < self.frame.data.shape[1] and y > 0 and y < self.frame.data.shape[0]])
-            for y, u, l in zip(ycoord, np.ceil(xcoord_edgeup).astype(int), np.floor(xcoord_edgelow).astype(int)):
-                self.frame.mask[y, l:u] = 1
+            axisBcoord = orderPixelTable.loc[
+                (orderPixelTable["order"] == o)][f"{self.axisB}coord"]
+            axisAcoord_edgeup = orderPixelTable.loc[(orderPixelTable["order"] == o)][
+                f"{self.axisA}coord_edgeup"] + expandEdges
+            axisAcoord_edgelow = orderPixelTable.loc[(orderPixelTable["order"] == o)][
+                f"{self.axisA}coord_edgelow"] - expandEdges
+            axisAcoord_edgelow, axisAcoord_edgeup, axisBcoord = zip(*[(x1, x2, b) for x1, x2, b in zip(axisAcoord_edgelow, axisAcoord_edgeup, axisBcoord) if x1 > 0 and x1 < axisALen and x2 > 0 and x2 < axisALen and b > 0 and b < axisBLen])
+            for b, u, l in zip(axisBcoord, np.ceil(axisAcoord_edgeup).astype(int), np.floor(axisAcoord_edgelow).astype(int)):
+                if self.axisA == "x":
+                    self.frame.mask[b, l:u] = 1
+                else:
+                    self.frame.mask[l:u, b] = 1
 
         self.log.debug('completed the ``mask_order_locations`` method')
         return None
@@ -183,10 +197,17 @@ class subtract_background(object):
         self.log.debug('starting the ``create_background_image`` method')
 
         from astropy.stats import sigma_clip, mad_std
+        import numpy as np
+        import pandas as pd
+        from astropy.nddata import CCDData
+        from scipy.signal import medfilt2d
+        from scipy.interpolate import BSpline, splrep
+        import numpy.ma as ma
+
         maskedImage = np.ma.array(self.frame.data, mask=self.frame.mask)
         # SIGMA-CLIP THE DATA
         clippedDataMask = sigma_clip(
-            maskedImage, sigma_lower=2, sigma_upper=5, maxiters=3, cenfunc='median', stdfunc=mad_std)
+            maskedImage, sigma_lower=2, sigma_upper=5, maxiters=3, cenfunc='median', stdfunc='mad_std')
         # COMBINE MASK WITH THE BAD PIXEL MASK
         mask = (clippedDataMask.mask == 1) | (self.frame.mask == 1)
         maskedImage = np.ma.array(self.frame.data, mask=mask)
@@ -198,7 +219,7 @@ class subtract_background(object):
         backgroundMap = np.zeros_like(self.frame)
 
         if self.surfaceFit:
-
+            import matplotlib.pyplot as plt
             fig = plt.figure(figsize=(40, 10))
             from scipy.interpolate import griddata
             flatMask = mask.flatten()
@@ -255,6 +276,7 @@ class subtract_background(object):
                 backgroundMap[idx, :] = yfit
 
                 if random.randint(1, 501) == 42 and 1 == 0:
+                    import matplotlib.pyplot as plt
                     fig, (ax1) = plt.subplots(1, 1, figsize=(30, 15))
                     plt.scatter(xmasked, rowmasked)
                     plt.scatter(xmasked, rowmaskedSmoothed)
