@@ -17,20 +17,21 @@ from copy import copy
 from datetime import datetime
 from soxspipe.commonutils import keyword_lookup
 import math
-import pandas as pd
-import numpy.ma as ma
+
+
 import random
-import matplotlib as mpl
-import matplotlib.pyplot as plt
-import numpy as np
+
+
 import unicodecsv as csv
 from soxspipe.commonutils.polynomials import chebyshev_xy_polynomial, chebyshev_order_xy_polynomials
 from fundamentals import tools
 from builtins import object
-from matplotlib import cm, rc
+
 import sys
+from soxspipe.commonutils.dispersion_map_to_pixel_arrays import dispersion_map_to_pixel_arrays
 import os
-from mpl_toolkits.mplot3d import Axes3D
+
+
 os.environ['TERM'] = 'vt100'
 
 
@@ -41,6 +42,7 @@ def cut_image_slice(
         length,
         x,
         y,
+        sliceAxis="x",
         median=False,
         plot=False):
     """*cut and return an N-pixel wide and M-pixels long slice, centred on a given coordinate from an image frame*
@@ -53,6 +55,7 @@ def cut_image_slice(
     - ``length`` -- length of the slice
     - ``x`` -- x-coordinate
     - ``y`` -- y-coordinate
+    - ``sliceAxis`` -- the axis along which slice is to be taken. Default *x*
     - ``median`` -- collapse the slice to a median value across its width
     - ``plot`` -- generate a plot of slice. Useful for debugging.
 
@@ -68,6 +71,9 @@ def cut_image_slice(
     """
     log.debug('starting the ``cut_image_slice`` function')
 
+    import numpy.ma as ma
+    import numpy as np
+
     halfSlice = length / 2
     # NEED AN EVEN PIXEL SIZE
     if (width % 2) != 0:
@@ -75,27 +81,59 @@ def cut_image_slice(
     else:
         halfwidth = width / 2
 
-    # CHECK WE ARE NOT GOING BEYOND BOUNDS OF FRAME
-    if (x > frame.shape[1] - halfSlice) or (y > frame.shape[0] - halfwidth) or (x < halfSlice) or (y < halfwidth):
-        return None
+    if sliceAxis == "x":
+        axisA = x
+        axisB = y
+        axisALen = frame.shape[1]
+        axisBLen = frame.shape[0]
+    elif sliceAxis == "y":
+        axisB = x
+        axisA = y
+        axisALen = frame.shape[0]
+        axisBLen = frame.shape[1]
+    else:
+        raise ValueError("sliceAxis needs to be eith 'x' or 'y'")
 
-    slice = frame[int(y - halfwidth):int(y + halfwidth + 1),
-                  int(x - halfSlice):int(x + halfSlice)]
+    # CHECK WE ARE NOT GOING BEYOND BOUNDS OF FRAME
+    if (axisA > axisALen - halfSlice) or (axisB > axisBLen - halfwidth) or (axisA < halfSlice) or (axisB < halfwidth):
+        return None, None, None
+
+    slice_length_offset = int(axisA - halfSlice)
+    if sliceAxis == "x":
+        sliceFull = frame[int(axisB - halfwidth):int(axisB + halfwidth + 1),
+                          slice_length_offset:int(axisA + halfSlice)]
+    else:
+        sliceFull = frame[slice_length_offset:int(axisA + halfSlice), int(axisB - halfwidth):int(axisB + halfwidth + 1)]
+    slice_width_centre = (int(axisB + halfwidth + 1) + int(axisB - halfwidth)) / 2
 
     if median:
-        slice = ma.median(slice, axis=0)
+        if sliceAxis == "y":
+            slice = ma.median(sliceFull, axis=1)
+        else:
+            slice = ma.median(sliceFull, axis=0)
 
     if plot and random.randint(1, 101) < 10000:
+        import matplotlib.pyplot as plt
         # CHECK THE SLICE POINTS IF NEEDED
-        x = np.arange(0, len(slice))
+        if sliceAxis == "y":
+            sliceImg = np.rot90(sliceFull, 1)
+        else:
+            sliceImg = sliceFull
+        plt.imshow(sliceImg)
+        plt.show()
+        xx = np.arange(0, len(slice))
         plt.figure(figsize=(8, 5))
-        plt.plot(x, slice, 'ko')
+        if sliceAxis == "y":
+            plt.plot(xx, slice, 'ko', label=f"x={axisB}, y={axisA}, sliceAxis={sliceAxis}")
+        if sliceAxis == "x":
+            plt.plot(xx, slice, 'ko', label=f"x={axisA}, y={axisB}, sliceAxis={sliceAxis}")
         plt.xlabel('Position')
         plt.ylabel('Flux')
+        plt.legend()
         plt.show()
 
     log.debug('completed the ``cut_image_slice`` function')
-    return slice
+    return slice, slice_length_offset, slice_width_centre
 
 
 def quicklook_image(
@@ -105,7 +143,10 @@ def quicklook_image(
         ext="data",
         stdWindow=3,
         title=False,
-        surfacePlot=False):
+        surfacePlot=False,
+        dispMap=False,
+        dispMapDF=False,
+        inst=False):
     """*generate a quicklook image of a CCDObject - useful for development/debugging*
 
     **Key Arguments:**
@@ -116,6 +157,9 @@ def quicklook_image(
     - ``ext`` -- the name of the the extension to show. Can be "data", "mask" or "err". Default "data".
     - ``title`` -- give a title for the plot
     - ``surfacePlot`` -- plot as a 3D surface plot
+    - ``dispMap`` -- path to dispersion map. Default *False*
+    - ``dispMapDF`` -- pandas dataframe of dispersion map wavelength, order and slit-postion
+    - ``inst`` -- provide instrument name if no header exists
 
     ```python
     from soxspipe.commonutils.toolkit import quicklook_image
@@ -125,10 +169,15 @@ def quicklook_image(
     """
     log.debug('starting the ``quicklook_image`` function')
 
+    import pandas as pd
+    import matplotlib as mpl
+    import numpy as np
+
     originalRC = dict(mpl.rcParams)
 
     if not show:
         return
+    import matplotlib.pyplot as plt
 
     if ext == "data":
         frame = CCDObject.data
@@ -140,8 +189,19 @@ def quicklook_image(
         # ASSUME ONLY NDARRAY
         frame = CCDObject
 
-    rotatedImg = np.rot90(frame, 1)
-    rotatedImg = np.flipud(np.rot90(frame, 1))
+    if inst is False:
+        try:
+            inst = CCDObject.header["INSTRUME"]
+        except:
+            inst = "XSHOOTER"
+
+    if inst == "SOXS":
+        rotatedImg = np.flipud(frame)
+    elif inst == "XSHOOTER":
+        rotatedImg = np.rot90(frame, 1)
+    else:
+        rotatedImg = frame
+    rotatedImg = np.flipud(rotatedImg)
 
     std = np.nanstd(frame)
     mean = np.nanmean(frame)
@@ -152,6 +212,8 @@ def quicklook_image(
 
     if surfacePlot:
 
+        from matplotlib import rc
+
         axisColour = '#dddddd'
         rc('axes', edgecolor=axisColour, labelcolor=axisColour, linewidth=0.6)
         rc('xtick', color=axisColour)
@@ -161,7 +223,8 @@ def quicklook_image(
 
         fig = plt.figure(figsize=(40, 10))
         ax = fig.add_subplot(121, projection='3d')
-        plt.gca().invert_yaxis()
+        if inst == "XSHOOTER":
+            plt.gca().invert_yaxis()
         ax.set_box_aspect(aspect=(2, 1, 1))
         # Remove gray panes and axis grid
         ax.xaxis.pane.fill = False
@@ -177,14 +240,18 @@ def quicklook_image(
         X, Y = np.meshgrid(np.linspace(0, rotatedImg.shape[1], rotatedImg.shape[1]), np.linspace(0, rotatedImg.shape[0], rotatedImg.shape[0]))
         surface = ax.plot_surface(X=X, Y=Y, Z=rotatedImg, cmap='viridis', antialiased=True, vmin=vmin, vmax=vmax)
 
-        ax.azim = -120
+        if inst == "SOXS":
+            ax.azim = 70
+        else:
+            ax.azim = -120
         ax.elev = 30
 
-        plt.gca().invert_xaxis()
         ax.set_xlim(0, rotatedImg.shape[1])
         ax.set_ylim(0, rotatedImg.shape[0])
-        ax.set_zlim(0, min(np.nanmax(frame), mean + stdWindow * 10 * std))
+        # ax.set_zlim(0, min(np.nanmax(frame), mean + stdWindow * 10 * std))
 
+        if inst == "SOXS":
+            ax.invert_yaxis()
         backgroundColour = '#404040'
         fig.set_facecolor(backgroundColour)
         ax.set_facecolor(backgroundColour)
@@ -192,10 +259,16 @@ def quicklook_image(
         ax.yaxis.pane.set_edgecolor(backgroundColour)
         ax.zaxis.pane.set_edgecolor(backgroundColour)
 
-        plt.xlabel(
-            "y-axis", fontsize=10)
-        plt.ylabel(
-            "x-axis", fontsize=10)
+        if inst == "SOXS":
+            plt.xlabel(
+                "x-axis", fontsize=10)
+            plt.ylabel(
+                "y-axis", fontsize=10)
+        else:
+            plt.xlabel(
+                "y-axis", fontsize=10)
+            plt.ylabel(
+                "x-axis", fontsize=10)
 
         ax2 = fig.add_subplot(122)
     else:
@@ -204,6 +277,51 @@ def quicklook_image(
         # palette.set_over('r', 1.0)
         # palette.set_under('g', 1.0)
         ax2 = fig.add_subplot(111)
+
+    if dispMap and not isinstance(dispMapDF, bool):
+        uniqueOrders = dispMapDF['order'].unique()
+        wlLims = []
+        spLims = []
+
+        for o in uniqueOrders:
+            filDF = dispMapDF.loc[dispMapDF["order"] == o]
+            wlLims.append((filDF['wavelength'].min(), filDF['wavelength'].max()))
+            spLims.append((filDF['slit_position'].min(), filDF['slit_position'].max()))
+
+        for o, wlLim, spLim in zip(uniqueOrders, wlLims, spLims):
+            wlRange = np.arange(wlLim[0], wlLim[1], 1)
+            wlRange = np.append(wlRange, [wlLim[1]])
+            for e in spLim:
+                myDict = {
+                    "order": np.full_like(wlRange, o),
+                    "wavelength": wlRange,
+                    "slit_position": np.full_like(wlRange, e)
+                }
+                orderPixelTable = pd.DataFrame(myDict)
+                orderPixelTable = dispersion_map_to_pixel_arrays(
+                    log=log,
+                    dispersionMapPath=dispMap,
+                    orderPixelTable=orderPixelTable
+                )
+                ax2.plot(orderPixelTable["fit_y"], orderPixelTable["fit_x"], "w:", alpha=0.5)
+            spRange = np.arange(spLim[0], spLim[1], 1)
+            spRange = np.append(spRange, [spLim[1]])
+            wlRange = np.arange(wlLim[0], wlLim[1], 15)
+            wlRange = np.append(wlRange, [wlLim[1]])
+            for l in wlRange:
+                myDict = {
+                    "order": np.full_like(spRange, o),
+                    "wavelength": np.full_like(spRange, l),
+                    "slit_position": spRange
+                }
+                orderPixelTable = pd.DataFrame(myDict)
+                orderPixelTable = dispersion_map_to_pixel_arrays(
+                    log=log,
+                    dispersionMapPath=dispMap,
+                    orderPixelTable=orderPixelTable
+                )
+                ax2.plot(orderPixelTable["fit_y"], orderPixelTable["fit_x"], "w:", alpha=0.5)
+
     ax2.set_box_aspect(0.5)
     detectorPlot = plt.imshow(rotatedImg, vmin=vmin, vmax=vmax,
                               cmap=palette, alpha=1, aspect='auto')
@@ -221,12 +339,19 @@ def quicklook_image(
         # plt.colorbar()
     if title:
         fig.suptitle(title, fontsize=16)
-    ax2.invert_yaxis()
+    if inst == "XSHOOTER":
+        ax2.invert_yaxis()
     # cbar.ticklabel_format(useOffset=False)
-    plt.xlabel(
-        "y-axis", fontsize=10)
-    plt.ylabel(
-        "x-axis", fontsize=10)
+    if inst == "SOXS":
+        plt.xlabel(
+            "x-axis", fontsize=10)
+        plt.ylabel(
+            "y-axis", fontsize=10)
+    else:
+        plt.xlabel(
+            "y-axis", fontsize=10)
+        plt.ylabel(
+            "x-axis", fontsize=10)
 
     plt.show()
     mpl.rcParams.update(originalRC)
@@ -238,13 +363,17 @@ def quicklook_image(
 def unpack_order_table(
         log,
         orderTablePath,
-        extend=0.):
+        extend=0.,
+        binx=1,
+        biny=1):
     """*unpack an order table and return a top-level `orderPolyTable` data-frame and a second `orderPixelTable` data-frame with the central-trace coordinates of each order given
 
     **Key Arguments:**
 
     - ``orderTablePath`` -- path to the order table
     - ``extend`` -- fractional increase to the order area in the y-axis (needed for masking)
+    - ``binx`` -- binning in the x-axis (from FITS header). Default *1*
+    - ``biny`` -- binning in the y-axis (from FITS header). Default *1*
 
     **Usage:**
 
@@ -257,7 +386,8 @@ def unpack_order_table(
     """
     log.debug('starting the ``functionName`` function')
     from astropy.table import Table
-
+    import pandas as pd
+    import numpy as np
     # MAKE RELATIVE HOME PATH ABSOLUTE
 
     home = expanduser("~")
@@ -269,72 +399,61 @@ def unpack_order_table(
     dat = Table.read(orderTablePath, format='fits', hdu=2)
     orderMetaTable = dat.to_pandas()
 
-    # ADD Y-COORD LIST
-    ycoords = [np.arange(math.floor(l) - int(r * extend), math.ceil(u) + int(r * extend), 1) for l, u, r in zip(
-        orderMetaTable["ymin"].values, orderMetaTable["ymax"].values, orderMetaTable["ymax"].values - orderMetaTable["ymin"].values)]
+    if "degy_cent" in orderPolyTable.columns:
+        axisA = "x"
+        axisB = "y"
+        axisAbin = binx
+        axisBbin = biny
+    else:
+        axisA = "y"
+        axisB = "x"
+        axisAbin = biny
+        axisBbin = binx
+
+    # ADD AXIS B COORD LIST
+    axisBcoords = [np.arange(0 if (math.floor(l) - int(r * extend)) < 0 else (math.floor(l) - int(r * extend)), 4200 if (math.ceil(u) + int(r * extend)) > 4200 else (math.ceil(u) + int(r * extend)), 1) for l, u, r in zip(
+        orderMetaTable[f"{axisB}min"].values, orderMetaTable[f"{axisB}max"].values, orderMetaTable[f"{axisB}max"].values - orderMetaTable[f"{axisB}min"].values)]
     orders = [np.full_like(a, o) for a, o in zip(
-        ycoords, orderMetaTable["order"].values)]
+        axisBcoords, orderMetaTable["order"].values)]
 
-    import pandas as pd
     # CREATE DATA FRAME FROM A DICTIONARY OF LISTS
     myDict = {
-        "ycoord": np.concatenate(ycoords),
+        f"{axisB}coord": np.concatenate(axisBcoords),
         "order": np.concatenate(orders)
     }
     orderPixelTable = pd.DataFrame(myDict)
 
-    # RUN COORDINATES THROUGH POLYNOMIALS TO GET X-COORDS
-    xcoords_centre = []
-    xcoords_edgeup = []
-    xcoords_edgelow = []
-
     cent_coeff = [float(v) for k, v in orderPolyTable.iloc[0].items() if "cent_" in k]
-    poly = chebyshev_order_xy_polynomials(log=log, y_deg=int(orderPolyTable.iloc[0]["degy_cent"]), order_deg=int(orderPolyTable.iloc[0]["degorder_cent"]), orderCol="order", yCol="ycoord").poly
-    orderPixelTable["xcoord_centre"] = poly(orderPixelTable, *cent_coeff)
+    poly = chebyshev_order_xy_polynomials(log=log, axisBCol=f"{axisB}coord", orderCol="order", orderDeg=int(orderPolyTable.iloc[0]["degorder_cent"]), axisBDeg=int(orderPolyTable.iloc[0][f"deg{axisB}_cent"])).poly
+    orderPixelTable[f"{axisA}coord_centre"] = poly(orderPixelTable, *cent_coeff)
 
-    if "degy_edgeup" in orderPolyTable.columns:
+    std_coeff = [float(v) for k, v in orderPolyTable.iloc[0].items() if "std_" in k]
+    if len(std_coeff):
+        poly = chebyshev_order_xy_polynomials(log=log, axisBDeg=int(orderPolyTable.iloc[0][f"deg{axisB}_cent"]), orderDeg=int(orderPolyTable.iloc[0]["degorder_cent"]), orderCol="order", axisBCol=f"{axisB}coord").poly
+        orderPixelTable["std"] = poly(orderPixelTable, *std_coeff)
+
+    if f"deg{axisB}_edgeup" in orderPolyTable.columns:
         upper_coeff = [float(v) for k, v in orderPolyTable.iloc[0].items() if "edgeup_" in k]
-        poly = chebyshev_order_xy_polynomials(log=log, y_deg=int(orderPolyTable.iloc[0]["degy_edgeup"]), order_deg=int(orderPolyTable.iloc[0]["degorder_edgeup"]), orderCol="order", yCol="ycoord").poly
-        orderPixelTable["xcoord_edgeup"] = poly(orderPixelTable, *upper_coeff)
+        poly = chebyshev_order_xy_polynomials(log=log, axisBDeg=int(orderPolyTable.iloc[0][f"deg{axisB}_edgeup"]), orderDeg=int(orderPolyTable.iloc[0]["degorder_edgeup"]), orderCol="order", axisBCol=f"{axisB}coord").poly
+        orderPixelTable[f"{axisA}coord_edgeup"] = poly(orderPixelTable, *upper_coeff)
 
-    if "degy_edgelow" in orderPolyTable.columns:
+    if f"deg{axisB}_edgelow" in orderPolyTable.columns:
         upper_coeff = [float(v) for k, v in orderPolyTable.iloc[0].items() if "edgelow_" in k]
-        poly = chebyshev_order_xy_polynomials(log=log, y_deg=int(orderPolyTable.iloc[0]["degy_edgelow"]), order_deg=int(orderPolyTable.iloc[0]["degorder_edgelow"]), orderCol="order", yCol="ycoord").poly
-        orderPixelTable["xcoord_edgelow"] = poly(orderPixelTable, *upper_coeff)
+        poly = chebyshev_order_xy_polynomials(log=log, axisBDeg=int(orderPolyTable.iloc[0][f"deg{axisB}_edgelow"]), orderDeg=int(orderPolyTable.iloc[0]["degorder_edgelow"]), orderCol="order", axisBCol=f"{axisB}coord").poly
+        orderPixelTable[f"{axisA}coord_edgelow"] = poly(orderPixelTable, *upper_coeff)
 
-    # for index, row in orderPolyTable.iterrows():
-    #     cent_coeff = [float(v) for k, v in row.items() if "cent_" in k]
-    #     poly = chebyshev_order_xy_polynomials(log=log, y_deg=int(row["degy_cent"]), order_deg=int(row["degorder_cent"])).poly
-    #     xcoords_centre.append(np.array(poly(ycoords[index], *cent_coeff)))
-    #     if "degy_edgeup" in row:
-    #         degy_edge = int(row["degy_edgeup"])
-    #         edgeup_coeff = [float(v)
-    #                         for k, v in row.items() if "edgeup_" in k]
-    #         poly = chebyshev_xy_polynomial(
-    #             log=log, y_deg=degy_edge).poly
-    #         xcoords_edgeup.append(
-    #             np.array(poly(ycoords[index], *edgeup_coeff)))
-
-    #         edgelow_coeff = [float(v)
-    #                          for k, v in row.items() if "edgelow_" in k]
-    #         poly = chebyshev_xy_polynomial(
-    #             log=log, y_deg=degy_edge).poly
-    #         xcoords_edgelow.append(
-    #             np.array(poly(ycoords[index], *edgelow_coeff)))
-
-    # # CREATE DATA FRAME FROM A DICTIONARY OF LISTS
-    # myDict = {
-    #     "order": np.concatenate(orders),
-    #     "ycoord": np.concatenate(ycoords),
-    #     'xcoord_centre': np.concatenate(xcoords_centre)
-    # }
-    # # ADD ODER EDGES IF NEEDED
-    # if len(xcoords_edgeup):
-    #     myDict['xcoord_edgeup'] = np.concatenate(xcoords_edgeup)
-    #     myDict['xcoord_edgelow'] = np.concatenate(xcoords_edgelow)
-
-    # # CREATE DATA FRAME FROM A DICTIONARY OF LISTS
-    # orderPixelTable = pd.DataFrame(myDict)
+    if axisAbin > 1:
+        orderPixelTable[f"{axisA}coord_centre"] /= axisAbin
+        orderPixelTable[f"{axisA}coord_edgeup"] /= axisAbin
+        orderPixelTable[f"{axisA}coord_edgelow"] /= axisAbin
+    if axisBbin > 1:
+        orderMetaTable[f"{axisB}min"] /= axisBbin
+        orderMetaTable[f"{axisB}max"] /= axisBbin
+        orderPixelTable[f"{axisB}coord"] /= axisBbin
+        orderPixelTable["std"] /= axisBbin
+        mask = (orderPixelTable[f"{axisB}coord"].mod(1) > 0)
+        orderPixelTable = orderPixelTable.loc[~mask]
+        orderPixelTable[f"{axisB}coord"] = orderPixelTable[f"{axisB}coord"].astype('int')
 
     log.debug('completed the ``functionName`` function')
     return orderPolyTable, orderPixelTable, orderMetaTable
@@ -371,6 +490,8 @@ def generic_quality_checks(
     """
     log.debug('starting the ``functionName`` function')
 
+    import numpy as np
+
     # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
     # FOLDER
     kw = keyword_lookup(
@@ -462,6 +583,9 @@ def spectroscopic_image_quality_checks(
     """
     log.debug('starting the ``functionName`` function')
 
+    import numpy.ma as ma
+    import numpy as np
+
     # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
     # FOLDER
     kw = keyword_lookup(
@@ -472,20 +596,36 @@ def spectroscopic_image_quality_checks(
     arm = frame.header[kw("SEQ_ARM")]
     dateObs = frame.header[kw("DATE_OBS")]
 
+    try:
+        binx = frame.header[kw("WIN_BINX")]
+        biny = frame.header[kw("WIN_BINY")]
+    except:
+        if arm.lower() == "nir":
+            binx = 1
+            biny = 1
+
+    inst = frame.header[kw("INSTRUME")]
+    if inst == "SOXS":
+        axisA = "y"
+        axisB = "x"
+    elif inst == "XSHOOTER":
+        axisA = "x"
+        axisB = "y"
+
     # UNPACK THE ORDER TABLE
     orderTableMeta, orderTablePixels, orderMetaTable = unpack_order_table(
-        log=log, orderTablePath=orderTablePath)
+        log=log, orderTablePath=orderTablePath, binx=binx, biny=biny)
 
     mask = np.ones_like(frame.data)
 
-    xcoords_up = orderTablePixels["xcoord_edgeup"].values
-    xcoords_low = orderTablePixels["xcoord_edgelow"].values
-    ycoords = orderTablePixels["ycoord"].values
-    xcoords_up = xcoords_up.astype(int)
-    xcoords_low = xcoords_low.astype(int)
+    axisACoords_up = orderTablePixels[f"{axisA}coord_edgeup"].values
+    axisACoords_low = orderTablePixels[f"{axisA}coord_edgelow"].values
+    axisBCoords = orderTablePixels[f"{axisB}coord"].values
+    axisACoords_up = axisACoords_up.astype(int)
+    axisACoords_low = axisACoords_low.astype(int)
 
     # UPDATE THE MASK
-    for u, l, y in zip(xcoords_up, xcoords_low, ycoords):
+    for u, l, y in zip(axisACoords_up, axisACoords_low, axisBCoords):
         mask[y][l:u] = 0
 
     # COMBINE MASK WITH THE BAD PIXEL MASK
@@ -600,7 +740,7 @@ def get_calibrations_path(
     ```python
     from soxspipe.commonutils.toolkit import get_calibrations_path
     calibrationRootPath = get_calibrations_path(log=log, settings=settings)
-    ```           
+    ```
     """
     log.debug('starting the ``get_calibrations_path`` function')
 
@@ -615,5 +755,90 @@ def get_calibrations_path(
     log.debug('completed the ``get_calibrations_path`` function')
     return calibrationRootPath
 
+
+def twoD_disp_map_image_to_dataframe(
+        log,
+        slit_length,
+        twoDMapPath,
+        assosiatedFrame=False,
+        removeMaskedPixels=False):
+    """*convert the 2D dispersion image map to a pandas dataframe*
+
+    **Key Arguments:**
+
+    - `log` -- logger
+    - `twoDMapPath` -- 2D dispersion map image path
+    - `assosiatedFrame` -- include a flux column in returned dataframe from a frame assosiated with the dispersion map. Default *False*
+    - `removeMaskedPixels` -- remove the masked pixels from the assosicated image? Default *False*
+
+    **Usage:**
+
+    ```python
+    from soxspipe.commonutils.toolkit import twoD_disp_map_image_to_dataframe
+    mapDF = twoD_disp_map_image_to_dataframe(log=log, twoDMapPath=twoDMap, assosiatedFrame=objectFrame)
+    ```           
+    """
+    log.debug('starting the ``twoD_disp_map_image_to_dataframe`` function')
+
+    import pandas as pd
+    import numpy as np
+    from astropy.io import fits
+
+    # MAKE RELATIVE HOME PATH ABSOLUTE
+    from os.path import expanduser
+    home = expanduser("~")
+    if twoDMapPath[0] == "~":
+        twoDMapPath = twoDMapPath.replace("~", home)
+
+    hdul = fits.open(twoDMapPath)
+
+    # MAKE X, Y ARRAYS TO THEN ASSOCIATE WITH WL, SLIT AND ORDER
+    xdim = hdul[0].data.shape[1]
+    ydim = hdul[0].data.shape[0]
+
+    xarray = np.tile(np.arange(0, xdim), ydim)
+    yarray = np.repeat(np.arange(0, ydim), xdim)
+
+    thisDict = {
+        "x": xarray,
+        "y": yarray,
+        "wavelength": hdul["WAVELENGTH"].data.flatten().byteswap().newbyteorder(),
+        "slit_position": hdul["SLIT"].data.flatten().byteswap().newbyteorder(),
+        "order": hdul["ORDER"].data.flatten().byteswap().newbyteorder(),
+    }
+
+    try:
+        if assosiatedFrame:
+            thisDict["flux"] = assosiatedFrame.data.flatten()
+            thisDict["mask"] = assosiatedFrame.mask.flatten()
+            thisDict["error"] = assosiatedFrame.uncertainty.flatten()
+
+    except:
+        if assosiatedFrame:
+            thisDict["flux"] = assosiatedFrame.data.flatten().byteswap().newbyteorder()
+            thisDict["mask"] = assosiatedFrame.mask.flatten().byteswap().newbyteorder()
+            thisDict["error"] = assosiatedFrame.uncertainty.array.flatten().byteswap().newbyteorder()
+
+    mapDF = pd.DataFrame.from_dict(thisDict)
+    if removeMaskedPixels:
+        mask = (mapDF["mask"] == False)
+        mapDF = mapDF.loc[mask]
+
+    # REMOVE ZEROS
+    mask = (mapDF['wavelength'] == 0)
+    mapDF = mapDF.loc[~mask]
+
+    mapDF.dropna(how="all", subset=["wavelength", "slit_position", "order"], inplace=True)
+
+    # REMOVE FILTERED ROWS FROM DATA FRAME
+    mask = ((mapDF['slit_position'] < -slit_length / 2) | (mapDF['slit_position'] > slit_length / 2))
+    mapDF.drop(index=mapDF[mask].index, inplace=True)
+
+    # SORT BY COLUMN NAME
+    mapDF.sort_values(['wavelength'], inplace=True)
+
+    log.debug('completed the ``twoD_disp_map_image_to_dataframe`` function')
+    return mapDF
+
 # use the tab-trigger below for new function
 # xt-def-function
