@@ -12,12 +12,7 @@
 ################# GLOBAL IMPORTS ####################
 from soxspipe.commonutils import detect_continuum
 from soxspipe.commonutils import keyword_lookup
-import ccdproc
-from astropy import units as u
-from astropy.nddata import CCDData
-import numpy as np
 from ._base_recipe_ import _base_recipe_
-from soxspipe.commonutils import set_of_files
 from fundamentals import tools
 from builtins import object
 from datetime import datetime
@@ -36,6 +31,7 @@ class soxs_order_centres(_base_recipe_):
         - ``settings`` -- the settings dictionary
         - ``inputFrames`` -- input fits frames. Can be a directory, a set-of-files (SOF) file or a list of fits frame paths.
         - ``verbose`` -- verbose. True or False. Default *False*
+        - ``overwrite`` -- overwrite the prodcut file if it already exists. Default *False*
 
     **Usage**
 
@@ -63,28 +59,30 @@ class soxs_order_centres(_base_recipe_):
             log,
             settings=False,
             inputFrames=[],
-            verbose=False
+            verbose=False,
+            overwrite=False
 
     ):
         # INHERIT INITIALISATION FROM  _base_recipe_
         super(soxs_order_centres, self).__init__(
-            log=log, settings=settings)
+            log=log, settings=settings, inputFrames=inputFrames, overwrite=overwrite, recipeName="soxs-order-centre")
         self.log = log
         log.debug("instansiating a new 'soxs_order_centres' object")
         self.settings = settings
         self.inputFrames = inputFrames
         self.verbose = verbose
-        self.recipeName = "soxs-order-centre"
         self.recipeSettings = settings[self.recipeName]
         # xt-self-arg-tmpx
 
         # INITIAL ACTIONS
         # CONVERT INPUT FILES TO A CCDPROC IMAGE COLLECTION (inputFrames >
         # imagefilecollection)
+        from soxspipe.commonutils.set_of_files import set_of_files
         sof = set_of_files(
             log=self.log,
             settings=self.settings,
-            inputFrames=self.inputFrames
+            inputFrames=self.inputFrames,
+            ext=self.settings['data-extension']
         )
         self.inputFrames, self.supplementaryInput = sof.get()
 
@@ -96,7 +94,7 @@ class soxs_order_centres(_base_recipe_):
         print("# VERIFYING INPUT FRAMES - ALL GOOD")
 
         # SORT IMAGE COLLECTION
-        self.inputFrames.sort(['mjd-obs'])
+        self.inputFrames.sort(['MJD-OBS'])
         if self.verbose:
             print("# RAW INPUT FRAMES - SUMMARY")
             print(self.inputFrames.summary, "\n")
@@ -167,6 +165,9 @@ class soxs_order_centres(_base_recipe_):
         """
         self.log.debug('starting the ``produce_product`` method')
 
+        from astropy.nddata import CCDData
+        from astropy import units as u
+
         arm = self.arm
         kw = self.kw
         dp = self.detectorParams
@@ -177,13 +178,13 @@ class soxs_order_centres(_base_recipe_):
         dark = False
         orderDef_image = False
 
-        add_filters = {kw("DPR_CATG"): 'MASTER_BIAS_' + arm}
+        add_filters = {kw("PRO_CATG"): 'MASTER_BIAS_' + arm}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
             master_bias = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                        hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
 
         # UVB/VIS DARK
-        add_filters = {kw("DPR_CATG"): 'MASTER_DARK_' + arm}
+        add_filters = {kw("PRO_CATG"): 'MASTER_DARK_' + arm}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
             dark = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                 hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
@@ -224,23 +225,41 @@ class soxs_order_centres(_base_recipe_):
         if self.settings["save-intermediate-products"]:
             fileDir = self.intermediateRootPath
             filepath = self._write(
-                self.orderFrame, fileDir, filename=False, overwrite=True)
+                self.orderFrame, fileDir, filename=False, overwrite=True, product=False)
             print(f"\nCalibrated single pinhole frame frame saved to {filepath}\n")
 
+        # FIND THE APPROPRIATE PREDICTED LINE-LIST
+        if arm != "NIR" and kw('WIN_BINX') in self.orderFrame.header:
+            binx = int(self.orderFrame.header[kw('WIN_BINX')])
+            biny = int(self.orderFrame.header[kw('WIN_BINY')])
+        else:
+            binx = 1
+            biny = 1
+
         # DETECT THE CONTINUUM OF ORDERE CENTRES - RETURN ORDER TABLE FILE PATH
+        print("\n# DETECTING ORDER CENTRE CONTINUUM\n")
         detector = detect_continuum(
             log=self.log,
             pinholeFlat=self.orderFrame,
             dispersion_map=disp_map_table,
             settings=self.settings,
-            recipeName="soxs-order-centre"
+            recipeName="soxs-order-centre",
+            qcTable=self.qc,
+            productsTable=self.products,
+            sofName=self.sofName,
+            binx=binx,
+            biny=biny
         )
-        productPath = detector.get()
+        productPath, qcTable, productsTable = detector.get()
+
+        self.products = self.products.append(productsTable)
+        self.qc = self.qc.append(qcTable)
 
         filename = os.path.basename(productPath)
 
         utcnow = datetime.utcnow()
         utcnow = utcnow.strftime("%Y-%m-%dT%H:%M:%S")
+        self.dateObs = self.orderFrame.header[kw("DATE_OBS")]
 
         self.products = self.products.append({
             "soxspipe_recipe": self.recipeName,
